/***************************************************************************//**
* @file     lcd_8D3B.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

//******************************************************************************
// Variable
//******************************************************************************

static uint8_t*         lcdfbuf;

static const uint8_t    xLCD_BUF[30] =
{
   0x7C, 0x12, 0x11, 0x12, 0x7C,    // "A"   0AH
   0x41, 0x7F, 0x49, 0x49, 0x36,    // "B"   0BH
   0x3E, 0x41, 0x41, 0x41, 0x22,    // "C"   0CH
   0x41, 0x7F, 0x41, 0x41, 0x3E,    // "D"   0DH
   0x7F, 0x49, 0x49, 0x49, 0x41,    // "E"   0EH
   0x7F, 0x09, 0x09, 0x09, 0x01,    // "F"   0FH
};

//******************************************************************************
// Function
//******************************************************************************

static void lcd_setting( void )
{
   char              i;
   LCD_CFG_Type      LCD_Config;

   // select peripheral clock: WDTRC
   HAL_SCU_Peripheral_ClockSelection( PPCLKSR_LCDCLK, LCDCLK_WDTRC );

   // init lcd
   LCD_Config.Bias = LCD_RLCD2;
   LCD_Config.Duty = LCD_8D4B;
   LCD_Config.Clk = LCD_CLK512;
   LCD_Config.AutoBiasEn = LCD_ABCEN;
   LCD_Config.BiasTime = LCD_BIA6CLK;
   LCD_Config.Contrast = LCD_CONTEN;
   LCD_Config.ContrastStep = LCD_VLC22;
   HAL_LCD_Init( &LCD_Config );

   // start address of Display Data RAM
   lcdfbuf = ( uint8_t* )( 0x40005010 );

   // init Display Data RAM
   for( i = 0; i < LCDBufSize; i++ )
   {
      lcdfbuf[i] = 0x00;
   }
}

static void l_init( void )
{
   int      i;

   for( i = 0; i < LCDBufSize; i++ )
   {
      lcdfbuf[i] = 0x00;
   }
}

static void l_disp( unsigned int disp )
{
   uint8_t     i;

   switch( disp % 4 )
   {
      case 0:
         for( i = 0; i < 10; i++ )
         {
            lcdfbuf[5 + i] = xLCD_BUF[( disp % 3 ) * 10 + i];
         }
         break;
      case 1:
         for( i = 0; i < 10; i++ )
         {
            lcdfbuf[5 + i] = 0xAA;;
         }
         break;
      case 2:
         for( i = 0; i < 10; i++ )
         {
            lcdfbuf[5 + i] = 0x55;
         }
         break;
      case 3:
         l_init();
         break;
      default:
         break;
   }
}

void t_lcd_8D3B( void )
{
   uint8_t     count = 0;

   DI();
   lcd_setting();
   l_init();
   EI();

   _DBG( "\n\rLCD 1/8duty 1/3bias test Program\n\r" );
   LCDON();

   while( 1 )
   {
      l_disp( count++ );

      msec = 500;
      while( msec );
   }
}

