/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void EINT0_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void PWR_DeepSleepRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "PWR_DeepSleep Example \n\r"
                        "  - MCU:        A31G112 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:                           System Control Block (SCB) \n\r"
                        "GPIO Output Pin:                             PE1 \n\r"
                        "GPIO Input Pin for Deep Sleep Mode Entrance: PB1 \n\r"
                        "EINT Input Pin for Deep Sleep Mode Exit:     PB0(EINT0) \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    test;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         EINT0_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void EINT0_IRQHandler_IT( void )
{
   //uint32_t    j;
   uint32_t    status;

   status = HAL_INT_EIntPB_GetIntStatus();
   if( status & ( _BIT( 0 ) ) )
   {
      //for (j= 0; j<5; j++)
      {
         if( test )
         {
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
            test = 0;
         }
         else
         {
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
            test = 1;
         }
      }
   }

   HAL_INT_EIntPB_ClearIntStatus( 0xFF );
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // Test Pin setting PE1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 1, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 1, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );

   // Test Pin setting PE2
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 2, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 2, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 2 ) );

   // external interrupt test pin PB0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, INPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_EnablePU );
   HAL_GPIO_SetDebouncePin( ( Pn_Type* )PB, 0, DBCLK_HCLK1024 );

   // external interrupt test pin PB1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, INPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_EnablePU );
   HAL_GPIO_SetDebouncePin( ( Pn_Type* )PB, 1, DBCLK_HCLK1024 );

   HAL_INT_EIntPx_SetReg( PORTB, 0, ITRIGx_Edge, INTCTLx_FallingEdge );

   HAL_INT_EIntCfg( EINT0, PORTB, 0 );

   HAL_INT_EIntPB_ClearIntStatus( 0xFF );
   NVIC_EnableIRQ( EINT0_IRQn );

   HAL_INT_EInt_MaskDisable( MSK_EINT0 );
}

/*-------------------------------------------------------------------------*//**
 * @brief         PWR_DeepSleepRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void PWR_DeepSleepRun( void )
{
   // wait user command
   _DBG( "Press PB1 to enter Deep Sleep Mode. \n\r\n\r" );
   while( HAL_GPIO_ReadPin( ( Pn_Type* )PB ) & ( 1uL <<  1 ) ) {}
   delay();

   // go to Deep Sleep Mode
   _DBG( "Press PB0 to exit Deep Sleep Mode. \n\r\n\r" );
   HAL_SCU_ClockMonitoring_Disable();
   HAL_PWR_EnterPowerDownMode();
   HAL_SCU_ClockMonitoring( MACTS_SysClkChg, MONCS_MCLK );

   // after wakeup
   _DBG( "Waked up from Deep Sleep Mode. \n\r\n\r" );

   /* Infinite loop */
   while( 1 )
   {
      // output high
      HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 1 ) );
      delay();

      // output low
      HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );
      delay();
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   // disable LVR (Low Voltage Reset by H/W, Option)
   SCULV_DisLVR();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* PWR_DeepSleepRun */
   PWR_DeepSleepRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

