/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void LVI_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void LVI_Configure( void );
void LVI_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "PWR_LVI_Interrupt Example \n\r"
                        "  - MCU:        A31G112 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: SCULV \n\r"
                        "GPIO Output Pin:   PE0 \n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    flag;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         LVI_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void LVI_IRQHandler_IT( void )
{
   if( SCULV_GetLviFlag() )
   {
      SCULV_ClrLviFlag();

      if( flag )
      {
         HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
         flag = 0;
      }
      else
      {
         HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
         flag = 1;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // Test Pin setting PE1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 1, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 1, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 1 ) );
}

/*-------------------------------------------------------------------------*//**
 * @brief         LVI_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void LVI_Configure( void )
{
   // LVI configuration
   {
      // lvi configuration
      {
         // init lvi: Interrupt Mode, 3.20V
         HAL_LVI_Init( LVIEN_Enable, LVINTEN_Enable, LVIVS_3p20V );
      }

      // unmask Interrupt
      NVIC_SetPriority( LVI_IRQn, 3 );
      NVIC_EnableIRQ( LVI_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_LVI );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         LVI_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void LVI_InterruptRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* LVI_Configure */
   LVI_Configure();

   // Low Voltage Reset by H/W (Option)
   SCULV_DisLVR();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* LVI_InterruptRun */
   LVI_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

