/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER20_Configure( void );
void TIMER2n_MatchPollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

#define Tx     TIMER20

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER2n_MatchPolling Example \n\r"
                        "  - MCU:        A31G112 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: TIMER20 \n\r"
                        "Using Timer 2n in polling mode\n\r"
                        "************************************************\n\r\n\r";

TIMER2n_PERIODICCFG_Type   TIMER2n_Config;
uint32_t                   count;
uint32_t                   flag;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // T20O pin config set if need
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 0, AFSRx_AF1 );
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, ALTERN_FUNC );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PC, 0, PUPDx_Disable );

   // T21O pin config set if need
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PC, 1, AFSRx_AF1 );
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 1, ALTERN_FUNC );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PC, 1, PUPDx_Disable );
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER20_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER20_Configure( void )
{
   count = 0;
   flag = 0;

   TIMER2n_Config.CkSel = TIMER2n_INTERNAL_CLK;       // PCLK = 40MHz
   TIMER2n_Config.Prescaler = 40;                     // fTIMER = PCLK / 40 = 1MHz
   TIMER2n_Config.ADR = ( 1000 );                     // Period  = ADR / fTIMER = 1000us
   TIMER2n_Config.BDR = ( 0 );                        // Duty(B) = BDR / fTIMER =    0us
   TIMER2n_Config.StartLevel = TIMER2n_START_LOW;
   TIMER2n_Config.TCLK_SEL = TIMER2n_PCLK_CLK;
   TIMER2n_Config.CapEdge = TIMER2n_FALLING_EGDE;     // dummy parameter
   TIMER2n_Config.ExtCkEdge = TIMER2n_FALLING_EGDE;   // dummy parameter

   HAL_TIMER2n_Init( ( TIMER2n_Type* )Tx, TIMER2n_PERIODIC_MODE, &TIMER2n_Config );
   HAL_TIMER2n_Cmd( ( TIMER2n_Type* )Tx, ENABLE );  // timer start
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER2n_MatchPollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER2n_MatchPollingRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
      while( ( HAL_TIMER2n_GetStatus( ( TIMER2n_Type* )Tx ) & TIMER2n_CR_MATCH_FLAG ) != TIMER2n_CR_MATCH_FLAG );
      HAL_TIMER2n_ClearStatus( ( TIMER2n_Type* )Tx, TIMER2n_CR_MATCH_FLAG );

      if( count++ >= 1000 )
      {
         count = 0;
         if( flag == 0 )
         {
            flag = 1;
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
         else
         {
            flag = 0;
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER20_Configure */
   TIMER20_Configure();

   /* TIMER2n_MatchPollingRun */
   TIMER2n_MatchPollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

