/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void TIMER30_IRQHandler_IT( void );
void ADC_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER30_Configure( void );
void ADC_Configure( void );
void TIMER3n_PWMRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Constant
//******************************************************************************

#define PWM_HZ       5000

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER3n_PWM Example \n\r"
                        "  - MCU:        A31G112 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: TIMER30 \n\r"
                        "Using Timer 30 in PWM mode\n\r"
                        "************************************************\n\r\n\r";

volatile uint32_t    count;
volatile uint32_t    count1;
volatile uint32_t    flag;
volatile uint32_t    flag1;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER30_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER30_IRQHandler_IT( void )
{
   if( ( HAL_TIMER3n_GetStatus_IT( ( TIMER3n_Type* )TIMER30 ) & TIMER3n_INT_PMATEN ) == TIMER3n_INT_PMATEN )
   {
      if( flag == 0 )
      {
         flag = 1;
         HAL_GPIO_ClearPin( ( Pn_Type* )PC, _BIT( 0 ) );
      }
      else
      {
         flag = 0;
         HAL_GPIO_SetPin( ( Pn_Type* )PC, _BIT( 0 ) );
      }
   }

   HAL_TIMER3n_ClearStatus_IT( ( TIMER3n_Type* )TIMER30, 0x7F );
}

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_IRQHandler_IT( void )
{
   uint16_t    status;

   status = HAL_ADC_GetStatus( ADC );

   if( status & ADC_STAT_END )
   {

      if( flag1 == 0 )
      {
         flag1 = 1;
         HAL_GPIO_ClearPin( ( Pn_Type* )PC, _BIT( 1 ) );
      }
      else
      {
         flag1 = 0;
         HAL_GPIO_SetPin( ( Pn_Type* )PC, _BIT( 1 ) );
      }

      HAL_ADC_ClearStatus( ADC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PC0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PC, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PC, _BIT( 0 ) );

   // Test Pin setting PC1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PC, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PC, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PC, _BIT( 1 ) );
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER30_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER30_Configure( void )
{
   // select peripheral clock: MCLK
   HAL_SCU_Peripheral_ClockSelection( PPCLKSR_T30CLK, T30CLK_MCLK );

   HAL_SCU_Peripheral_EnableClock1( PPCLKEN1_T30CLKE, PPxCLKE_Enable );

   HAL_TIMER3n_ClockPrescaler( ( TIMER3n_Type* )TIMER30, 0 );      // 40MHz/(0+1) = 40MHz

   HAL_TIMER3n_SetPeriod( ( TIMER3n_Type* )TIMER30, ( uint16_t )( ( ( uint32_t )40000000L / ( PWM_HZ ) ) / 2 - 1 ) );
   HAL_TIMER3n_SetADuty( ( TIMER3n_Type* )TIMER30, ( TIMER30->PDR >> 1 ) );
   HAL_TIMER3n_SetBDuty( ( TIMER3n_Type* )TIMER30, ( TIMER30->PDR >> 1 ) );
   HAL_TIMER3n_SetCDuty( ( TIMER3n_Type* )TIMER30, ( TIMER30->PDR >> 1 ) );

   HAL_TIMER3n_OutputCtrl( ( TIMER3n_Type* )TIMER30, ENABLE, TIMER3n_OUT_APOLOW, TIMER3n_OUT_BPOHIGH );

   HAL_TIMER3n_ConfigInterrupt( ( TIMER3n_Type* )TIMER30, ENABLE, TIMER3n_INT_PMATEN );
   HAL_TIMER3n_ClearStatus_IT( ( TIMER3n_Type* )TIMER30, 0x7F );

   HAL_TIMER3n_SetADCTrigger( ( TIMER3n_Type* )TIMER30, TIMER3n_ADT_BTTGEN, 0 );

   HAL_TIMER3n_MPWMCmd( ( TIMER3n_Type* )TIMER30, TIMER3n_UPMATCH, TIMER3n_E1PERIOD );

   HAL_TIMER3n_SetDelayTime( ( TIMER3n_Type* )TIMER30, TIMER3n_DLYINSEN, TIMER3n_INSFRONT, 79 );    // (79 + 1) / 40MHz = 2us

   HAL_TIMER3n_Start( ( TIMER3n_Type* )TIMER30, TIMER3n_ENABLE );

   NVIC_EnableIRQ( TIMER30_IRQn );
   HAL_INT_EInt_MaskDisable( MSK_TIMER30 );
}

/*-------------------------------------------------------------------------*//**
 * @brief         ADC_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void ADC_Configure( void )
{
   ADC_CFG_Type      ADC_Config;

   // ADC configuration
   {
      // adc configuration
      {
         ADC_Config.RefSel = ADC_REF_VDD;    // 0:ADC_REF_VDD ,1:ADC_REF_AVREF
         ADC_Config.TrgSel = ADC_TRIGGER_TIMER30;
         ADC_Config.InClkDiv = 14;           // 40M/14 = 2.8M (less than 3M)
         if( HAL_ADC_Init( ADC, &ADC_Config ) != HAL_OK )
         {
            /* Initialization Error */
            Error_Handler();
         }
         HAL_ADC_ConfigInterrupt( ADC, ENABLE );
      }

      // unmask Interrupt
      //NVIC_SetPriority(ADC_IRQn, 3);
      NVIC_EnableIRQ( ADC_IRQn );
      HAL_INT_EInt_MaskDisable( MSK_ADC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER3n_PWMRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER3n_PWMRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER30_Configure */
   TIMER30_Configure();    // motor pwm initial

   /* ADC_Configure */
   ADC_Configure();        // adc trigger mode by TIMER3n

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* TIMER3n_PWMRun */
   TIMER3n_PWMRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

