/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void UST_IntTransmit( USART1n_Type* USART1x );
void UST_IntReceive( USART1n_Type* USART1x );
void UST_SPI_SS_control( USART1n_Type* USART1x, char ch );
void USART10_IRQHandler_IT( void );
void USART11_IRQHandler_IT( void );
void UART1_Configure( void );
void SPI10_Configure( void );
void SPI11_Configure( void );
void USART1n_SPI_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         UST_IntTransmit
 * @param[in]     USART1x
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UST_IntTransmit( USART1n_Type* USART1x )
{
   if( USART1x == ( USART1n_Type* )USART10 )
   {
      UST_SPI_SS_control( ( USART1n_Type* )USART10, 1 );
   }
   else if( USART1x == ( USART1n_Type* )USART11 )
   {
      UST_SPI_SS_control( ( USART1n_Type* )USART11, 1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UST_IntReceive
 * @param[in]     USART1x
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UST_IntReceive( USART1n_Type* USART1x )
{
   uint8_t     ch;

   ch = USART1x->DR;
   HAL_UART_Transmit( ( UARTn_Type* )UART1, &ch, 1, BLOCKING );
}

/*-------------------------------------------------------------------------*//**
 * @brief         UST_SPI_SS_control
 * @param[in]     USART1x
 * @param[in]     ch
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UST_SPI_SS_control( USART1n_Type* USART1x, char ch )
{
   if( USART1x == ( USART1n_Type* )USART10 )
   {
      if( ch == 0 )
      {
         PB->OUTDR_b.OUTDR3 = 0;
      }
      else
      {
         PB->OUTDR_b.OUTDR3 = 1;
      }
   }
   else if( USART1x == ( USART1n_Type* )USART11 )
   {
      if( ch == 0 )
      {
         PD->OUTDR_b.OUTDR5 = 0;
      }
      else
      {
         PD->OUTDR_b.OUTDR5 = 1;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART10_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART10_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   /* Determine the interrupt source */
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART10 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // Receive Data Available or Character time-out
   if( ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\n\rSPI10 rx : ", 13, BLOCKING );
      UST_IntReceive( ( USART1n_Type* )USART10 );
      HAL_USART_ClearStatus( ( USART1n_Type* )USART10, USART1n_STATUS_RXC );
   }

   // Transmit Holding Empty
   if( ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      UST_IntTransmit( ( USART1n_Type* )USART10 );
      HAL_USART_ClearStatus( ( USART1n_Type* )USART10, USART1n_STATUS_TXC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART11_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART11_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   /* Determine the interrupt source */
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART11 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // Receive Data Available or Character time-out
   if( ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\n\rSPI11 rx : ", 13, BLOCKING );
      UST_IntReceive( ( USART1n_Type* )USART11 );
      HAL_USART_ClearStatus( ( USART1n_Type* )USART11, USART1n_STATUS_RXC );
   }

   // Transmit Holding Empty
   if( ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      UST_IntTransmit( ( USART1n_Type* )USART11 );
      HAL_USART_ClearStatus( ( USART1n_Type* )USART11, USART1n_STATUS_TXC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART1_Configure( void )
{
   UARTn_CFG_Type    UARTn_Config;

   /*
   * Initialize UART1 pin connect
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 7, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 7, AFSRx_AF1 );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 7, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 6, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 6, AFSRx_AF1 );

   HAL_UART_ConfigStructInit( &UARTn_Config );
   HAL_UART_Init( ( UARTn_Type* )UART1, ( UARTn_CFG_Type* )( &UARTn_Config ) );

   HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\n\rUSART SPI Interrupt test \n\r", 29, BLOCKING );
   HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"SPI10 master, SPI11 slave\n\r", 27, BLOCKING );
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
   * Initialize USART10
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 0, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 1, AFSRx_AF2 );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 2, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );

   // default: 38400-8-N-1
   HAL_USART_SPI_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

   // SPI Master
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, ENABLE );

   UST_SPI_SS_control( ( USART1n_Type* )USART10, 1 ); // SS init

   // interrupt
   NVIC_SetPriority( USART10_IRQn, 3 );
   NVIC_EnableIRQ( USART10_IRQn );
   INTC->MSK |= ( 1 << USART10_IRQn ); // USART10 interrupt unmask

   HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART10, USART1n_INTCFG_TXC, ENABLE );
   //HAL_USART_ConfigInterrupt((USART1n_Type*)(USART1n_Type*)USART10, USART1n_INTCFG_RXC, ENABLE );

   HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI11_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI11_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
   * Initialize USART11
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 3, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 3, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 4, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 5, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 5, AFSRx_AF2 );

   // default: 38400-8-N-1
   HAL_USART_SPI_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART11, &USART1n_Config );

   // SPI Master
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_MASTER, DISABLE );

   // interrupt
   NVIC_SetPriority( USART11_IRQn, 3 );
   NVIC_EnableIRQ( USART11_IRQn );
   INTC->MSK |= ( 1 << USART11_IRQn ); // USART11 interrupt unmask

   //HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART11, USART1n_INTCFG_TXC, ENABLE );
   HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART11, USART1n_INTCFG_RXC, ENABLE );

   // Slave SS Function Enable
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_USTSSEN, ENABLE );

   // ENABLE
   HAL_USART_Enable( ( USART1n_Type* )USART11, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_SPI_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_SPI_InterruptRun( void )
{
   uint8_t     spiTx_ch;

   /* Infinite loop */
   while( 1 )
   {
      HAL_UART_Receive( ( UARTn_Type* )UART1, &spiTx_ch, 1, BLOCKING );

      //slave Tx
      USART11->DR = '#';

      //master Tx
      UST_SPI_SS_control( ( USART1n_Type* )USART10, 0 );
      USART10->DR = spiTx_ch;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* UART1_Configure */
   UART1_Configure();

   /* SPI10_Configure */
   SPI10_Configure();

   /* SPI11_Configure */
   SPI11_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* USART1n_SPI_InterruptRun */
   USART1n_SPI_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   //DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

