/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G11x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void UART10_Configure( void );
void USART1n_UART_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         UART10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
    * Initialize USART10 pin connect
    */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 1, AFSRx_AF1 );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 0, AFSRx_AF1 );

   // default: 38400-8-N-1
   HAL_USART_UART_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

   HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_UART_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_UART_PollingRun( void )
{
   uint8_t     ch;

   HAL_USART_Transmit( ( USART1n_Type* )USART10, ( uint8_t* )"USART10 Uart Polling test \n\r", sizeof( "USART10 Uart Polling test \n\r" ), BLOCKING );

   /* Infinite loop */
   while( 1 )
   {
      HAL_USART_Receive( ( USART1n_Type* )USART10, &ch, 1, BLOCKING );
      HAL_USART_Transmit( ( USART1n_Type* )USART10, &ch, 1, BLOCKING );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* UART10_Configure */
   UART10_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* USART1n_UART_PollingRun */
   USART1n_UART_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   //DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

