/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void SysTick_ExceptionHandler_IT( void );
void TIMER10_IRQHandler_IT( void );
void GPIO_Configure( void );
void TIMER10_Configure( void );
void DEMO_FNDRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

volatile uint32_t       msec;

TIMER1n_PWMCFG_Type     TIMER1n_Config;
volatile uint32_t       count;
volatile uint32_t       flag;

// Constants definition
const uint8_t     fnd_dsp_data[10] = {0x3F, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x27, 0x7f, 0x6f};     // 0~9
uint8_t           fnd6 = 0, fnd5 = 0, fnd4 = 0, fnd3 = 0, fnd2 = 0, fnd1 = 0, inc = 0;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         SysTick_ExceptionHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SysTick_ExceptionHandler_IT( void )
{
   if( msec > 0 )
   {
      msec--;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER10_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER10_IRQHandler_IT( void )
{
   uint32_t    status;

   status = T1nMaInt_GetFg( TIMER10 );
   if( status == 1 )
   {
      T1nMaInt_ClrFg( TIMER10 );

      HAL_GPIO_WritePin( ( Pn_Type* )PD, 0x3F ); //FND OFF

      if( inc == 0 )
      {
         HAL_GPIO_WritePin( ( Pn_Type* )PE, fnd_dsp_data[fnd1] );
         HAL_GPIO_WritePin( ( Pn_Type* )PD, 0xFE );
      }
      if( inc == 1 )
      {
         HAL_GPIO_WritePin( ( Pn_Type* )PE, fnd_dsp_data[fnd2] );
         HAL_GPIO_WritePin( ( Pn_Type* )PD, 0xFD );
      }
      if( inc == 2 )
      {
         HAL_GPIO_WritePin( ( Pn_Type* )PE, fnd_dsp_data[fnd3] );
         HAL_GPIO_WritePin( ( Pn_Type* )PD, 0xFB );
      }
      if( inc == 3 )
      {
         HAL_GPIO_WritePin( ( Pn_Type* )PE, fnd_dsp_data[fnd4] );
         HAL_GPIO_WritePin( ( Pn_Type* )PD, 0xF7 );
      }
      if( inc == 4 )
      {
         HAL_GPIO_WritePin( ( Pn_Type* )PE, fnd_dsp_data[fnd5] );
         HAL_GPIO_WritePin( ( Pn_Type* )PD, 0xEF );
      }
      if( inc == 5 )
      {
         HAL_GPIO_WritePin( ( Pn_Type* )PE, fnd_dsp_data[fnd6] );
         HAL_GPIO_WritePin( ( Pn_Type* )PD, 0xDF );
      }
      inc++;
      if( inc == 6 )
      {
         inc = 0;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PD0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 0, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 0 ) );

   // Test Pin setting PD1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 1, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 1, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 1 ) );

   // Test Pin setting PB2
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 2, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 2 ) );

   // Test Pin setting PD3
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 3, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 3, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 3 ) );

   // Test Pin setting PD4
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 4, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 4 ) );

   // Test Pin setting PD5
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 5, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 5, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 5 ) );

   // Test Pin setting PD6
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 6, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 6, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 6 ) );

   // Test Pin setting PD7
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 7, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PD, 7, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 7 ) );

   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // Test Pin setting PE1
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 1, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 1, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 1 ) );

   // Test Pin setting PB2
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 2, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 2, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 2 ) );

   // Test Pin setting PE3
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 3, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 3, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 3 ) );

   // Test Pin setting PE4
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 4, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 4, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 4 ) );

   // Test Pin setting PE5
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 5, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 5, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 5 ) );
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER10_Configure( void )
{
   TIMER1n_Config.CkSel = TIMER1n_PCLK;               // PCLK = 40MHz
   TIMER1n_Config.Prescaler = 40;                     // fTIMER = PCLK / 40 = 1MHz
   TIMER1n_Config.ADR = ( 1000 );                     // Period  = ADR / fTIMER = 1000us
   TIMER1n_Config.StartLevel = TIMER1n_START_LOW;

   HAL_TIMER1n_Init( ( TIMER1n_Type* )TIMER10, TIMER1n_PERIODIC_MODE, &TIMER1n_Config );
   HAL_TIMER1n_ConfigInterrupt( ( TIMER1n_Type* )TIMER10, TIMER1n_INTCFG_MIE, ENABLE );

   NVIC_SetPriority( TIMER10_IRQn, 3 );
   NVIC_EnableIRQ( TIMER10_IRQn );
   HAL_INT_EInt_MaskDisable( MSK_TIMER10 );

   HAL_TIMER1n_Cmd( ( TIMER1n_Type* )TIMER10, ENABLE );   // timer start & clear
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEMO_FNDRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEMO_FNDRun( void )
{
   while( 1 )
   {
      msec = 10;
      while( msec );
      fnd6++;
      if( fnd6 == 10 )
      {
         fnd6 = 0;
         ++fnd5;
         if( fnd5 == 10 )
         {
            fnd5 = 0;
            ++fnd4;
            if( fnd4 == 10 )
            {
               fnd4 = 0;
               ++fnd3;
               if( fnd3 == 10 )
               {
                  fnd3 = 0;
                  ++fnd2;
                  if( fnd2 == 10 )
                  {
                     fnd2 = 0;
                     ++fnd1;
                     if( fnd1 == 10 )
                     {
                        fnd1 = 0;
                     }
                  }
               }
            }
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER10_Configure */
   TIMER10_Configure();

   // configure SysTick to 1ms interrupt
   SysTick_Config( SystemCoreClock / 1000 );

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* DEMO_FNDRun */
   DEMO_FNDRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   //DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

