/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void delay( void );
void EINT0_IRQHandler_IT( void );
void EINT1_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void GPIO_ExtIntRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "GPIO_EINT Example \n\r"
                        "  - MCU:        A31G123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: INTC \n\r"
                        "This example used to configure the\n\r "
                        "GPIO PB0,11 as source of external Interrupt \n\r"
                        "GPIO PE0,11 as monitor port \n\r"
                        "************************************************\n\r\n\r";

uint8_t mode_level[] =  "\n\r\t - 0: Prohibit external Interrupt \n\r"
                        "\t - 1: low level Interrupt \n\r"
                        "\t - 2: high level Interrupt \n\r";

uint8_t mode_edge[] =   "\n\r\t - 0: Prohibit external Interrupt \n\r"
                        "\t - 1: falling edge Interrupt \n\r"
                        "\t - 2: rising edge Interrupt \n\r"
                        "\t - 3: both edge Interrupt \n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         delay
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void delay( void )
{
   uint32_t    i;

   for( i = 0; i < 0x100000; i++ );
}

/*-------------------------------------------------------------------------*//**
 * @brief         EINT0_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void EINT0_IRQHandler_IT( void )
{
   uint32_t    j;
   uint32_t    status;

   status = HAL_INT_EIntPB_GetIntStatus();
   if( status & ( _BIT( 0 ) ) )
   {
      for( j = 0; j < 5; j++ )
      {
         HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
         delay();
         HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
         delay();
      }
   }

   HAL_INT_EIntPB_ClearIntStatus( 0xFFFul );
}

/*-------------------------------------------------------------------------*//**
 * @brief         EINT1_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void EINT1_IRQHandler_IT( void )
{
   uint32_t    j;
   uint32_t    status;

   status = HAL_INT_EIntPB_GetIntStatus();
   if( status & ( _BIT( 11 ) ) )
   {
      for( j = 0; j < 5; j++ )
      {
         HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 11 ) );
         delay();
         HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 11 ) );
         delay();
      }
   }

   HAL_INT_EIntPB_ClearIntStatus( 0xFFFul );
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // Test Pin setting PE11
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 11, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 11, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 11 ) );

   // Test Pin setting PE2
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 2, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 2, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 2 ) );

   // external interrupt test pin PB0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, INPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 0, PUPDx_EnablePU );
   HAL_GPIO_SetDebouncePin( ( Pn_Type* )PB, 0, DBCLK_HCLK1024 );

   // external interrupt test pin PB11
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 11, INPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 11, PUPDx_EnablePU );
   HAL_GPIO_SetDebouncePin( ( Pn_Type* )PB, 11, DBCLK_HCLK1024 );
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_ExtIntRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_ExtIntRun( void )
{
   uint8_t     flag;
   uint8_t     val_level_or_edge, mode;

   // Get some inputs
   _DBG( "\n\rPlease input character 0:LEVEL or 1: EDGE! \n\r" );

   val_level_or_edge = 0;
   flag = 1;
   while( flag )
   {
      val_level_or_edge = _DG();
      switch( val_level_or_edge )
      {
         case '0':
            _DBC( val_level_or_edge );
            val_level_or_edge = ITRIGx_Level;
            flag = 0;
            break;
         case '1':
            _DBC( val_level_or_edge );
            val_level_or_edge = ITRIGx_Edge;
            flag = 0;
            break;
         default:
            _DBG( "\n\r...Please input character 0:LEVEL or 1: EDGE! \n\r" );
            break;
      }
   }

   if( val_level_or_edge == ITRIGx_Level )
   {
      _DBG( mode_level );
   }
   else
   {
      _DBG( mode_edge );
   }

   mode = 0;
   flag = 1;
   while( flag )
   {
      mode = _DG();
      if( val_level_or_edge == ITRIGx_Level )
      {
         switch( mode )
         {
            case '0':
            case '1':
            case '2':
               _DBC( mode );
               mode = mode - 0x30;
               flag = 0;
               break;
            default:
               _DBG( "\n\r...Please input digit from 0 to 2 only! \n\r" );
               break;
         }
      }
      else
      {
         switch( mode )
         {
            case '0':
            case '1':
            case '2':
            case '3':
               _DBC( mode );
               mode = mode - 0x30;
               flag = 0;
               break;
            default:
               _DBG( "\n\r...Please input digit from 0 to 3 only! \n\r" );
               break;
         }
      }
   }

   HAL_INT_EIntPx_SetReg( PORTB, 0, val_level_or_edge, mode );
   HAL_INT_EIntPx_SetReg( PORTB, 11, val_level_or_edge, mode );

   HAL_INT_EIntCfg( EINT0, PORTB, 0 );
   HAL_INT_EIntCfg( EINT1, PORTB, 11 );

   HAL_INT_EIntPB_ClearIntStatus( 0xFFF );
   NVIC_EnableIRQ( EINT0_IRQn );
   NVIC_EnableIRQ( EINT1_IRQn );

   HAL_INT_EInt_MaskDisable( MSK_EINT0 | MSK_EINT1 );

   /* Infinite loop */
   while( 1 )
   {
      delay();
      HAL_GPIO_SetPin( ( Pn_Type* )PD, _BIT( 1 ) );
      delay();
      HAL_GPIO_ClearPin( ( Pn_Type* )PD, _BIT( 1 ) );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* GPIO_ExtIntRun */
   GPIO_ExtIntRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

