/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void GPIO_inputRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "GPIO_Input Example \n\r"
                        "  - MCU:        A31G123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: PCU \n\r"
                        "GPIO Input test \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_inputRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_inputRun( void )
{
   uint8_t     flag;
   uint8_t     port, pin, tmp;
   Pn_Type*    Px;

   // Get some inputs
   _DBG( "\n\rPlease enter PORT number (from A to F) \n\r" );

   port = 0;
   flag = 1;
   while( flag )
   {
      port = _DG();
      switch( port )
      {
         case 'A':
            _DBC( port );
            Px = ( Pn_Type* )PA;
            flag = 0;
            break;
         case 'B':
            _DBC( port );
            Px = ( Pn_Type* )PB;
            flag = 0;
            break;
         case 'C':
            _DBC( port );
            Px = ( Pn_Type* )PC;
            flag = 0;
            break;
         case 'D':
            _DBC( port );
            Px = ( Pn_Type* )PD;
            flag = 0;
            break;
         case 'E':
            _DBC( port );
            Px = ( Pn_Type* )PE;
            flag = 0;
            break;
         case 'F':
            _DBC( port );
            Px = ( Pn_Type* )PF;
            flag = 0;
            break;
         default:
            _DBG( "\n\r...Please input character from A to F only! \n\r" );
            break;
      }
   }

   flag = 0;
   tmp = 0;
   pin = 0;

   while( flag < 1 )
   {
      if( flag == 0 )
      {
         _DBG( "\n\rPlease enter PIN number (from 0 to 9):\n\r" );
      }
      tmp = _DG();
      switch( tmp )
      {
         case '0':
         case'1':case '2':case '3':case '4':case '5':case '6':case '7':case '8':case '9':
               pin = tmp - 0x30;
            _DBD( pin );
            tmp = 0;
            flag = 1;
            break;
         default:
            _DBG( "\n\r...Please input digits from 0 to 9 only! \n\r" );
            flag = 0;
            pin = 0;
            tmp = 0;
            break;
      }
   }

   HAL_GPIO_ConfigOutput( Px, pin, INPUT );
   HAL_GPIO_ConfigPullup( Px, pin, PUPDx_EnablePU );

   /* Infinite loop */
   while( 1 )
   {
      if( HAL_GPIO_ReadPin( Px ) & ( 1 << pin ) )
      {
         HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
      }
      else
      {
         HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* GPIO_inputRun */
   GPIO_inputRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

