/***************************************************************************//**
* @file     lcd.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

//******************************************************************************
// Variable
//******************************************************************************

static uint8_t*      lcdfbuf;

static const char    LEDSEG[17] =
{
   // 0    1    2    3    4    5    6    7    8    9    A    b    c    d    E    F    ' '
   // 0x0f,0x00,0x0d,0x09,0x02,0x0b,0x0f,0x03,0x0f,0x0b,0x07,0x0e,0x0f,0x0c,0x0f,0x07,0x00
   0x0f, 0x00, 0x0b, 0x09, 0x04, 0x0d, 0x0f, 0x0c, 0x0f, 0x0d, 0x0e, 0x07, 0x0f, 0x03, 0x0f, 0x0e, 0x00
};

static const char    LEDSEGn[17] =
{
   // 0    1    2    3    4    5    6    7    8    9    A    b    c    d    E    F    ' '
   // 0x05,0x05,0x03,0x07,0x07,0x06,0x06,0x05,0x07,0x07,0x07,0x06,0x00,0x07,0x02,0x02,0x00
   0x0a, 0x0a, 0x0c, 0x0e, 0x0e, 0x06, 0x06, 0x0a, 0x0e, 0x0e, 0x0e, 0x06, 0x00, 0x0e, 0x0e, 0x0e, 0x00
};

static char          ring_state;

//******************************************************************************
// Function
//******************************************************************************

static void lcd_setting( void )
{
   char              i;
   LCD_CFG_Type      LCD_Config;

   // select peripheral clock: WDTRC
   HAL_SCU_Peripheral_ClockSelection( PPCLKSR_LCDCLK, LCDCLK_WDTRC );

   // init lcd
   LCD_Config.Bias = LCD_RLCD2;
   LCD_Config.Duty = LCD_4D3B;
   LCD_Config.Clk = LCD_CLK128;
   LCD_Config.AutoBiasEn = LCD_ABCDIS;
   LCD_Config.BiasTime = LCD_BIA4CLK;
   LCD_Config.Contrast = LCD_CONTDIS;
   LCD_Config.ContrastStep = LCD_VLC31;
   HAL_LCD_Init( &LCD_Config );

   // start address of Display Data RAM
   lcdfbuf = ( uint8_t* )( 0x40005010 );

   // init Display Data RAM
   for( i = 0; i < 27; i++ )
   {
      lcdfbuf[i] = 0x00;
   }
}

static void l_init( void )
{
   int      i;

   for( i = 5; i < 14; i++ )
   {
      lcdfbuf[i] = 0;
   }
}

static void l_disp( unsigned int disp )
{
   char     tch[6];

   tch[0] = disp % 10;
   disp /= 10;
   if( disp == 0 )
   {
      tch[1] = tch[2] = tch[3] = 16;
   }
   else
   {
      tch[1] = disp % 10;
      disp /= 10;
      if( disp == 0 )
      {
         tch[2] = tch[3] = 16;
      }
      else
      {
         tch[2] = disp % 10;
         disp /= 10;
         if( disp == 0 )
         {
            tch[3] = 16;
         }
         else
         {
            tch[3] = disp % 10;
         }
      }
   }

   lcdfbuf[5] = LEDSEG[tch[3]];
   lcdfbuf[6] = LEDSEGn[tch[3]];
   lcdfbuf[7] = LEDSEG[tch[2]];
   lcdfbuf[8] = LEDSEGn[tch[2]];
   lcdfbuf[9] = LEDSEG[tch[1]];
   lcdfbuf[10] = LEDSEGn[tch[1]];
   lcdfbuf[11] = LEDSEG[tch[0]];
   lcdfbuf[12] = LEDSEGn[tch[0]];
}

static void l_ring( char dir )
{
   char     tch[8];

   if( dir )
   {
      ring_state++;
      if( ring_state > 11 )
      {
         ring_state = 0;
      }
   }
   else
   {
      ring_state--;
      if( ring_state == 0xff )
      {
         ring_state = 11;
      }
   }

   switch( ring_state )
   {
      case  0:
         tch[0] = 0x08;
         tch[1] = 0   ;
         tch[2] = 0x08;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case  1:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0x08;
         tch[3] = 0   ;
         tch[4] = 0x08;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case  2:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0x08;
         tch[5] = 0   ;
         tch[6] = 0x08;
         tch[7] = 0   ;
         break;
      case  3:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0x08;
         tch[7] = 0x08;
         break;
      case  4:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0x0a;
         break;
      case  5:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0x01;
         tch[7] = 0x02;
         break;
      case  6:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0x01;
         tch[5] = 0   ;
         tch[6] = 0x01;
         tch[7] = 0   ;
         break;
      case  7:
         tch[0] = 0   ;
         tch[1] = 0   ;
         tch[2] = 0x01;
         tch[3] = 0   ;
         tch[4] = 0x01;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case  8:
         tch[0] = 0x01;
         tch[1] = 0   ;
         tch[2] = 0x01;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case  9:
         tch[0] = 0x03;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case 10:
         tch[0] = 0x06;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      case 11:
         tch[0] = 0x0c;
         tch[1] = 0   ;
         tch[2] = 0   ;
         tch[3] = 0   ;
         tch[4] = 0   ;
         tch[5] = 0   ;
         tch[6] = 0   ;
         tch[7] = 0   ;
         break;
      default:
         break;
   }

   DI();
   lcdfbuf[5] = tch[0];
   lcdfbuf[6] = tch[1];
   lcdfbuf[7] = tch[2];
   lcdfbuf[8] = tch[3];
   lcdfbuf[9] = tch[4];
   lcdfbuf[10] = tch[5];
   lcdfbuf[11] = tch[6];
   lcdfbuf[12] = tch[7];
   lcdfbuf[13] = 0;
   EI();
}

static void l_bar( void )
{
   DI();
   lcdfbuf[5] = 0;
   lcdfbuf[6] = 0x04;
   lcdfbuf[7] = 0;
   lcdfbuf[8] = 0x04;
   lcdfbuf[9] = 0;
   lcdfbuf[10] = 0x04;
   lcdfbuf[11] = 0;
   lcdfbuf[12] = 0x04;
   lcdfbuf[13] = 0;
   EI();
}

static void lcd_test( void )
{
   uint8_t        ch;
   char           cmd;
   unsigned int   count = 0;

   cmd = '2';

   LCDON();

   for( ;; )
   {
      if( _DG_( &ch ) )
      {
         cmd = ch;
         _DBC( ch );
      }

      switch( cmd )
      {
         case '1':
            DI();
            l_init();
            EI();
            cmd = 'x';
            break;
         case '2':
            l_disp( count++ );
            msec = 500;
            while( msec );
            break;
         case '3':
            l_ring( 0 );
            msec = 80;
            while( msec );
            count = 0;
            break;
         case '4':
            l_ring( 1 );
            msec = 120;
            while( msec );
            count = 0;
            break;
         case '5':
            l_bar();
            msec = 120;
            while( msec );
            count = 0;
            break;
         case '6':
            msec = 600;
            while( msec );
            count = 0;
            break;
         case '7':
            msec = 800;
            while( msec );
            count = 0;
            break;
         case 'q':
         case 'Q':
            return;
         default:
            break;
      }
   }
}

void t_lcd( void )
{
   char     cmd;

   DI();
   lcd_setting();
   EI();

   while( 1 )
   {
      _DBG( "\n\rLCD test Program\n\r" );
      _DBG( "1. LCD OFF\n\r" );
      _DBG( "2. LCD ON/Count Number\n\r" );
      _DBG( "3. LCD Ring Rotation Left\n\r" );
      _DBG( "4. LCD Ring Rotation Right\n\r" );
      _DBG( "5. LCD Center Bar\n\r" );
      _DBG( "q. return to main.\n\r" );

      _DBG( ">" );
      lcd_test();

      getstring();
      if( InFlag )
      {
         _DBG( "\n\r" );
         cmd = InData[0];

         switch( cmd )
         {
            case '1':
               LCDON();
               break;
            case '2':
               LCDOFF();
               break;
            case '3':
               lcd_test();
               break;
            case 'q':
            case 'Q':
               _DBG( "Bye~\n\r" );
               return;
         }
      }
      else
      {
         _DBG( "\n\r" );
         InFlag = 0;
         InCount = 0;
      }
   }
}

