/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void TIMER15_IRQHandler_IT( void );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void GPIO_Configure( void );
void TIMER15_Configure( void );
void TIMER1n_MatchInterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "TIMER1n_MatchInterrupt Example \n\r"
                        "  - MCU:        A31G123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral: TIMER15 \n\r"
                        "Using Timer15 in interrupt mode \n\r"
                        "************************************************\n\r\n\r";

TIMER1n_PERIODICCFG_Type      TIMER1n_Config;
volatile uint32_t             count;
volatile uint32_t             flag;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER15_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER15_IRQHandler_IT( void )
{
   uint32_t    status;

   status = T1nMaInt_GetFg( TIMER15 );
   if( status == 1 )
   {
      T1nMaInt_ClrFg( TIMER15 );
      count++;
      if( count >= 100 )
      {
         count = 0;
         if( flag == 0 )
         {
            flag = 1;
            HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
         else
         {
            flag = 0;
            HAL_GPIO_SetPin( ( Pn_Type* )PE, _BIT( 0 ) );
         }
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         GPIO_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void GPIO_Configure( void )
{
   // Test Pin setting PE0
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 0, PUSH_PULL_OUTPUT );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 0, PUPDx_Disable );
   HAL_GPIO_ClearPin( ( Pn_Type* )PE, _BIT( 0 ) );

   // T15O pin config set if need
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 11, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 11, AFSRx_AF1 );
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER15_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER15_Configure( void )
{
   count = 0;
   flag = 0;

   TIMER1n_Config.CkSel = TIMER1n_PCLK;               // PCLK = 40MHz
   TIMER1n_Config.Prescaler = 40;                     // fTIMER = PCLK / 40 = 1MHz
   TIMER1n_Config.ADR = ( 1000 );                     // Period  = ADR / fTIMER = 1000us
   TIMER1n_Config.StartLevel = TIMER1n_START_LOW;
   TIMER1n_Config.BDR = 0;                            // Duty(B) = BDR / fTIMER =    0us  // dummy parameter
   TIMER1n_Config.ECE = TIMER1n_FALLING_EGDE;         // dummy parameter

   HAL_TIMER1n_Init( ( TIMER1n_Type* )TIMER15, TIMER1n_PERIODIC_MODE, &TIMER1n_Config );
   HAL_TIMER1n_ConfigInterrupt( ( TIMER1n_Type* )TIMER15, TIMER1n_INTCFG_MIE, ENABLE );

   NVIC_SetPriority( TIMER15_IRQn, 3 );
   NVIC_EnableIRQ( TIMER15_IRQn );
   HAL_INT_EInt_MaskDisable( MSK_TIMER15 );

   HAL_TIMER1n_Cmd( ( TIMER1n_Type* )TIMER15, ENABLE );   // timer start & clear
}

/*-------------------------------------------------------------------------*//**
 * @brief         TIMER1n_MatchInterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void TIMER1n_MatchInterruptRun( void )
{
   /* Infinite loop */
   while( 1 )
   {
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* Configure port peripheral */
   GPIO_Configure();

   /* TIMER15_Configure */
   TIMER15_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* TIMER1n_MatchInterruptRun */
   TIMER1n_MatchInterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

