/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void UST_IntTransmit( USART1n_Type* USART1x );
void UST_IntReceive( USART1n_Type* USART1x );
void UST_SPI_SS_control( USART1n_Type* USART1x, char ch );
void USART12_IRQHandler_IT( void );
void USART13_IRQHandler_IT( void );
void UART1_Configure( void );
void SPI12_Configure( void );
void SPI13_Configure( void );
void USART1n_SPI_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         UST_IntTransmit
 * @param[in]     USART1x
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UST_IntTransmit( USART1n_Type* USART1x )
{
   if( USART1x == ( USART1n_Type* )USART10 )
   {
      UST_SPI_SS_control( ( USART1n_Type* )USART10, 1 );
   }
   else if( USART1x == ( USART1n_Type* )USART11 )
   {
      UST_SPI_SS_control( ( USART1n_Type* )USART11, 1 );
   }
   else if( USART1x == ( USART1n_Type* )USART12 )
   {
      UST_SPI_SS_control( ( USART1n_Type* )USART12, 1 );
   }
   else if( USART1x == ( USART1n_Type* )USART13 )
   {
      UST_SPI_SS_control( ( USART1n_Type* )USART13, 1 );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UST_IntReceive
 * @param[in]     USART1x
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UST_IntReceive( USART1n_Type* USART1x )
{
   uint8_t     ch;

   ch = USART1x->DR;
   HAL_UART_Transmit( ( UARTn_Type* )UART1, &ch, 1, BLOCKING );
}

/*-------------------------------------------------------------------------*//**
 * @brief         UST_SPI_SS_control
 * @param[in]     USART1x
 * @param[in]     ch
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UST_SPI_SS_control( USART1n_Type* USART1x, char ch )
{
   if( USART1x == ( USART1n_Type* )USART10 )
   {
      if( ch == 0 )
      {
         PB->OUTDR_b.OUTDR3 = 0;
      }
      else
      {
         PB->OUTDR_b.OUTDR3 = 1;
      }
   }
   else if( USART1x == ( USART1n_Type* )USART11 )
   {
      if( ch == 0 )
      {
         PD->OUTDR_b.OUTDR5 = 0;
      }
      else
      {
         PD->OUTDR_b.OUTDR5 = 1;
      }
   }
   else if( USART1x == ( USART1n_Type* )USART12 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR15 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR15 = 1;
      }
   }
   else if( USART1x == ( USART1n_Type* )USART13 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR11 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR11 = 1;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART12_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART12_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   /* Determine the interrupt source */
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART12 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // Receive Data Available or Character time-out
   if( ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\n\rSPI10 rx : ", 13, BLOCKING );
      UST_IntReceive( ( USART1n_Type* )USART12 );
      HAL_USART_ClearStatus( ( USART1n_Type* )USART12, USART1n_STATUS_RXC );
   }

   // Transmit Holding Empty
   if( ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      UST_IntTransmit( ( USART1n_Type* )USART12 );
      HAL_USART_ClearStatus( ( USART1n_Type* )USART12, USART1n_STATUS_TXC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART13_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART13_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   /* Determine the interrupt source */
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART13 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // Receive Data Available or Character time-out
   if( ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\n\rSPI11 rx : ", 13, BLOCKING );
      UST_IntReceive( ( USART1n_Type* )USART13 );
      HAL_USART_ClearStatus( ( USART1n_Type* )USART13, USART1n_STATUS_RXC );
   }

   // Transmit Holding Empty
   if( ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      UST_IntTransmit( ( USART1n_Type* )USART13 );
      HAL_USART_ClearStatus( ( USART1n_Type* )USART13, USART1n_STATUS_TXC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART1_Configure( void )
{
   UARTn_CFG_Type    UARTn_Config;

   /*
   * Initialize UART1 pin connect
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 7, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 7, AFSRx_AF1 );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 7, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 6, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 6, AFSRx_AF1 );

   HAL_UART_ConfigStructInit( &UARTn_Config );
   HAL_UART_Init( ( UARTn_Type* )UART1, ( UARTn_CFG_Type* )( &UARTn_Config ) );

   HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\n\rUSART SPI Interrupt test \n\r", 29, BLOCKING );
   HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"SPI12 master, SPI13 slave\n\r", 27, BLOCKING );
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI12_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI12_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
   * Initialize USART12
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 12, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 12, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 13, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 13, AFSRx_AF2 );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PE, 13, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 14, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 14, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 15, PUSH_PULL_OUTPUT );

   // default: 38400-8-N-1
   HAL_USART_SPI_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART12, &USART1n_Config );

   // SPI Master
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART12, USART1n_CONTROL_MASTER, ENABLE );

   UST_SPI_SS_control( ( USART1n_Type* )USART12, 1 ); // SS init

   // interrupt
   NVIC_SetPriority( USART12_IRQn, 3 );
   NVIC_EnableIRQ( USART12_IRQn );
   INTC->MSK |= ( 1 << USART12_IRQn ); // USART12 interrupt unmask

   HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART12, USART1n_INTCFG_TXC, ENABLE );
   //HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART12, USART1n_INTCFG_RXC, ENABLE );

   HAL_USART_Enable( ( USART1n_Type* )USART12, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI13_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI13_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
   * Initialize USART13
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 8, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 8, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 9, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 9, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 10, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 10, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 11, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 11, AFSRx_AF2 );

   // default: 38400-8-N-1
   HAL_USART_SPI_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART13, &USART1n_Config );

   // SPI Master
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART13, USART1n_CONTROL_MASTER, DISABLE );

   // interrupt
   NVIC_SetPriority( USART13_IRQn, 3 );
   NVIC_EnableIRQ( USART13_IRQn );
   INTC->MSK |= ( 1 << USART13_IRQn ); // USART13 interrupt unmask

   //HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART13, USART1n_INTCFG_TXC, ENABLE );
   HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART13, USART1n_INTCFG_RXC, ENABLE );

   // Slave SS Function Enable
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART13, USART1n_CONTROL_USTSSEN, ENABLE );

   // ENABLE
   HAL_USART_Enable( ( USART1n_Type* )USART13, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_SPI_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_SPI_InterruptRun( void )
{
   uint8_t     spiTx_ch;

   /* Infinite loop */
   while( 1 )
   {
      HAL_UART_Receive( ( UARTn_Type* )UART1, &spiTx_ch, 1, BLOCKING );

      //slave Tx
      USART13->DR = '#';

      //master Tx
      UST_SPI_SS_control( ( USART1n_Type* )USART12, 0 );
      USART12->DR = spiTx_ch;
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* UART1_Configure */
   UART1_Configure();

   /* SPI12_Configure */
   SPI12_Configure();

   /* SPI13_Configure */
   SPI13_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* USART1n_SPI_InterruptRun */
   USART1n_SPI_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   //DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

