/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void UST_SPI_SS_control( USART1n_Type* USART1x, char ch );
void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void SPI10_Configure( void );
void SPI11_Configure( void );
void USART1n_SPI_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "USART1n_SPI_Polling Example \n\r"
                        "  - MCU:        A31G123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:   SPI10 Master \n\r"
                        "Counterpart:         SPI11 Slave \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         UST_SPI_SS_control
 * @param[in]     USART1x
 * @param[in]     ch
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UST_SPI_SS_control( USART1n_Type* USART1x, char ch )
{
   if( USART1x == ( USART1n_Type* )USART10 )
   {
      if( ch == 0 )
      {
         PB->OUTDR_b.OUTDR3 = 0;
      }
      else
      {
         PB->OUTDR_b.OUTDR3 = 1;
      }
   }
   else if( USART1x == ( USART1n_Type* )USART11 )
   {
      if( ch == 0 )
      {
         PD->OUTDR_b.OUTDR5 = 0;
      }
      else
      {
         PD->OUTDR_b.OUTDR5 = 1;
      }
   }
   else if( USART1x == ( USART1n_Type* )USART12 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR15 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR15 = 1;
      }
   }
   else if( USART1x == ( USART1n_Type* )USART13 )
   {
      if( ch == 0 )
      {
         PE->OUTDR_b.OUTDR11 = 0;
      }
      else
      {
         PE->OUTDR_b.OUTDR11 = 1;
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
   * Initialize USART10
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 0, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 1, AFSRx_AF2 );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 1, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 2, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 3, PUSH_PULL_OUTPUT );

   // default: 38400-8-N-1
   HAL_USART_SPI_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

   // SPI Master
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, ENABLE );


   UST_SPI_SS_control( ( USART1n_Type* )USART10, 1 ); // SS init
   HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         SPI11_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void SPI11_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
   * Initialize USART11
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 3, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 3, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 4, AFSRx_AF2 );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 5, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 5, AFSRx_AF2 );

   // default: 38400-8-N-1
   HAL_USART_SPI_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART11, &USART1n_Config );

   // SPI Master
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_MASTER, DISABLE );

   // Slave SS Function Enable
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_USTSSEN, ENABLE );

   // ENABLE
   HAL_USART_Enable( ( USART1n_Type* )USART11, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_SPI_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_SPI_PollingRun( void )
{
   uint8_t     spiTx_ch = 0, spiRx_ch = 0;

   /* Infinite loop */
   while( 1 )
   {
      HAL_UART_Receive( ( UARTn_Type* )UART1, &spiTx_ch, 1, BLOCKING );

      HAL_USART_TransmitByte( ( USART1n_Type* )USART11, '#' );

      UST_SPI_SS_control( ( USART1n_Type* )USART10, 0 );
      HAL_USART_Transmit( ( USART1n_Type* )USART10, &spiTx_ch, 1, BLOCKING );
      UST_SPI_SS_control( ( USART1n_Type* )USART10, 1 );

      HAL_USART_Receive( ( USART1n_Type* )USART11, &spiRx_ch, 1, BLOCKING );
      spiTx_ch = HAL_USART_ReceiveByte( ( USART1n_Type* )USART10 );

      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\r\nSPI10 rx:", 11, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, &spiTx_ch, 1, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )" SPI11 rx:", 10, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, &spiRx_ch, 1, BLOCKING );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* SPI10_Configure */
   SPI10_Configure();

   /* SPI11_Configure */
   SPI11_Configure();

   /* USART1n_SPI_PollingRun */
   USART1n_SPI_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

