/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void USART11_IRQHandler_IT( void );
void USART12_IRQHandler_IT( void );
void UART1_Configure( void );
void USRT11_Configure( void );
void USRT12_Configure( void );
void USART1n_USRT_InterruptRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

uint32_t    fflag0;
uint32_t    fflag1;
uint32_t    rx_data0;
uint32_t    rx_data1;

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         USART11_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART11_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   /* Determine the interrupt source */
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART11 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // Receive Data Available or Character time-out
   if( ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      rx_data1 = HAL_USART_ReceiveByte( ( USART1n_Type* )USART11 );
      USART11->DR = rx_data1;    // echo
      fflag1 = 1;
   }

   // Transmit Holding Empty
   if( ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      HAL_USART_ClearStatus( ( USART1n_Type* )USART11, USART1n_STATUS_TXC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART12_IRQHandler_IT
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART12_IRQHandler_IT( void )
{
   uint32_t    intsrc, tmp;

   /* Determine the interrupt source */
   intsrc = HAL_USART_GetStatus( ( USART1n_Type* )USART12 );
   tmp = intsrc & USART1n_SR_BITMASK;

   // Receive Data Available or Character time-out
   if( ( tmp & USART1n_SR_RXC ) == USART1n_SR_RXC )
   {
      rx_data0 = HAL_USART_ReceiveByte( ( USART1n_Type* )USART12 );
      fflag0 = 1;
   }

   // Transmit Holding Empty
   if( ( tmp & USART1n_SR_TXC ) == USART1n_SR_TXC )
   {
      HAL_USART_ClearStatus( ( USART1n_Type* )USART12, USART1n_STATUS_TXC );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         UART1_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void UART1_Configure( void )
{
   UARTn_CFG_Type    UARTn_Config;

   /*
   * Initialize UART1 pin connect
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 7, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 7, AFSRx_AF1 );
   HAL_GPIO_ConfigPullup( ( Pn_Type* )PB, 7, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 6, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 6, AFSRx_AF1 );

   HAL_UART_ConfigStructInit( &UARTn_Config );
   HAL_UART_Init( ( UARTn_Type* )UART1, ( UARTn_CFG_Type* )( &UARTn_Config ) );

   HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"USART USRT test \n\r", sizeof( "USART USRT test \n\r" ), BLOCKING );
}

/*-------------------------------------------------------------------------*//**
 * @brief         USRT11_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USRT11_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
   * Initialize USART11
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, AFSRx_AF1 );
   //HAL_GPIO_ConfigPullup( (Pn_Type*)PD, 2, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 3, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 3, AFSRx_AF1 );
   //HAL_GPIO_ConfigPullup( (Pn_Type*)PD, 3, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 4, AFSRx_AF2 );

   // default: 38400-8-N-1
   HAL_USART_USRT_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART11, &USART1n_Config );

   // USRT Slave
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_MASTER, DISABLE );
   HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART11, USART1n_INTCFG_TXC, ENABLE );
   HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART11, USART1n_INTCFG_RXC, ENABLE );

   NVIC_SetPriority( USART12_IRQn, 3 );
   NVIC_EnableIRQ( USART12_IRQn );
   HAL_INT_EInt_MaskDisable( MSK_USART12 );

   NVIC_SetPriority( USART11_IRQn, 3 );
   NVIC_EnableIRQ( USART11_IRQn );
   HAL_INT_EInt_MaskDisable( MSK_USART11 );

   // ENABLE
   HAL_USART_Enable( ( USART1n_Type* )USART11, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         USRT12_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USRT12_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   /*
   * Initialize USART12
   */
   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 12, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 12, AFSRx_AF1 );
   //HAL_GPIO_ConfigPullup( (Pn_Type*)PE, 12, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 13, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 13, AFSRx_AF1 );
   //HAL_GPIO_ConfigPullup( (Pn_Type*)PE, 13, PUPDx_EnablePU );

   HAL_GPIO_ConfigOutput( ( Pn_Type* )PE, 14, ALTERN_FUNC );
   HAL_GPIO_ConfigFunction( ( Pn_Type* )PE, 14, AFSRx_AF2 );

   // default: 38400-8-N-1
   HAL_USART_USRT_Mode_Config( &USART1n_Config );
   HAL_USART_Init( ( USART1n_Type* )USART12, &USART1n_Config );

   // USRT Master
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART12, USART1n_CONTROL_MASTER, ENABLE );
   HAL_USART_DataControlConfig( ( USART1n_Type* )USART12, USART1n_CONTROL_DISSCK, ENABLE );
   HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART12, USART1n_INTCFG_TXC, ENABLE );
   HAL_USART_ConfigInterrupt( ( USART1n_Type* )USART12, USART1n_INTCFG_RXC, ENABLE );

   // ENABLE
   HAL_USART_Enable( ( USART1n_Type* )USART12, ENABLE );
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_USRT_InterruptRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_USRT_InterruptRun( void )
{
   uint8_t     usrtTx_ch = 0;

   USART11->DR = '#';

   /* Infinite loop */
   while( 1 )
   {
      HAL_UART_Receive( ( UARTn_Type* )UART1, &usrtTx_ch, 1, BLOCKING );
      USART12->DR = usrtTx_ch;

      while( fflag0 == 0 );
      fflag0 = 0;
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\n\rUST12 rx :", 12, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )&rx_data0, 1, BLOCKING );

      while( fflag1 == 0 );
      fflag1 = 0;
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\n\rUST11 rx :", 12, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )&rx_data1, 1, BLOCKING );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* UART1_Configure */
   UART1_Configure();

   /* USRT11_Configure */
   USRT11_Configure();

   /* USRT12_Configure */
   USRT12_Configure();

   /* Enable IRQ Interrupts */
   __enable_irq();

   /* USART1n_USRT_InterruptRun */
   USART1n_USRT_InterruptRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   //DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

