/***************************************************************************//**
* @file     main.c
* @brief    Contains all functions support for Example Code on A31G12x
*
*
*
* @version  1.00
* @date     2020-05-29
* @author   ABOV Application Team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*//****************************************************************************/

//******************************************************************************
// Include
//******************************************************************************

#include "main_conf.h"

/* Private typedef ---------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private macro ------------------------------------------------------------ */
/* Private variables -------------------------------------------------------- */
/* Private define ----------------------------------------------------------- */
/* Private function prototypes ---------------------------------------------- */

void DEBUG_Init( void );
void DEBUG_MenuPrint( void );
void USRT10_Configure( void );
void USRT11_Configure( void );
void USART1n_USRT_PollingRun( void );
void mainloop( void );
int main( void );
void Error_Handler( void );

/* Private variables -------------------------------------------------------- */
//******************************************************************************
// Variable
//******************************************************************************

const uint8_t menu[] =  "************************************************\n\r"
                        "USART1n_USRT_Polling Example \n\r"
                        "  - MCU:        A31G123 \n\r"
                        "  - Core:       Cortex-M0+ \n\r"
                        "  - Monitoring: UART1 - 38400-8-N-1 \n\r"
                        "Target Peripheral:     USRT10 Master \n\r"
                        "Counterpart:           USRT11 Slave \n\r"
                        "************************************************\n\r\n\r";

//******************************************************************************
// Function
//******************************************************************************

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_Init
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_Init( void )
{
#ifdef _DEBUG_MSG
   debug_frmwrk_init();
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         DEBUG_MenuPrint
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void DEBUG_MenuPrint( void )
{
#ifdef _DEBUG_MSG
   _DBG( menu );
#endif
}

/*-------------------------------------------------------------------------*//**
 * @brief         USRT10_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USRT10_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // USRT10 configuration
   {
      // usrt TXD/RXD/SCK pin configuration
      {
         // print
         _DBG( "PB0 was configured as a TXD10. \n\r" );
         _DBG( "PB1 was configured as a RXD10. \n\r" );
         _DBG( "PB2 was configured as a SCK10. \n\r\n\r" );

         // configure PB0 as a TXD10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 0, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 0, AFSRx_AF1 );    // PB0            0: SEG41       1: TXD10       2: MOSI10      3: AN8         4: ----

         // configure PB1 as a RXD10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 1, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 1, AFSRx_AF1 );    // PB1            0: SEG40       1: RXD10       2: MISO10      3: AN9         4: ----

         // configure PB2 as a SCK10
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PB, 2, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PB, 2, AFSRx_AF2 );    // PB2            0: SEG39       1: ----        2: SCK10       3: AN10        4: ----
      }

      // usrt master mode configuration
      {
         // get default USART1n_Config for usrt: 38400-8-N-1
         HAL_USART_USRT_Mode_Config( &USART1n_Config );

         // init usart1n
         HAL_USART_Init( ( USART1n_Type* )USART10, &USART1n_Config );

         // enable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_MASTER, ENABLE );

         // select clock waveform
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART10, USART1n_CONTROL_DISSCK, ENABLE );

         // enable usart1n
         HAL_USART_Enable( ( USART1n_Type* )USART10, ENABLE );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USRT11_Configure
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USRT11_Configure( void )
{
   USART1n_CFG_Type     USART1n_Config;

   // USRT11 configuration
   {
      // usrt TXD/RXD/SCK pin configuration
      {
         // print
         _DBG( "PD2 was configured as a TXD11. \n\r" );
         _DBG( "PD3 was configured as a RXD11. \n\r" );
         _DBG( "PD4 was configured as a SCK11. \n\r\n\r" );

         // configure PD2 as a TXD11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 2, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 2, AFSRx_AF1 );    // PD2            0: SEG10       1: TXD11       2: MOSI11      3: ----        4: ----

         // configure PD3 as a RXD11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 3, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 3, AFSRx_AF1 );    // PD3            0: SEG9        1: RXD11       2: MISO11      3: ----        4: ----

         // configure PD4 as a SCK11
         HAL_GPIO_ConfigOutput( ( Pn_Type* )PD, 4, ALTERN_FUNC );
         HAL_GPIO_ConfigFunction( ( Pn_Type* )PD, 4, AFSRx_AF2 );    // PD4            0: SEG8        1: ----        2: SCK11       3: ----        4: ----
      }

      // usrt slave mode configuration
      {
         // get default USART1n_Config for usrt: 38400-8-N-1
         HAL_USART_USRT_Mode_Config( &USART1n_Config );

         // init usart1n
         HAL_USART_Init( ( USART1n_Type* )USART11, &USART1n_Config );

         // disable master
         HAL_USART_DataControlConfig( ( USART1n_Type* )USART11, USART1n_CONTROL_MASTER, DISABLE );

         // enable usart1n
         HAL_USART_Enable( ( USART1n_Type* )USART11, ENABLE );
      }
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         USART1n_USRT_PollingRun
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void USART1n_USRT_PollingRun( void )
{
   uint8_t     spiTx_ch = 0, spiRx_ch = 0;

   /* Infinite loop */
   while( 1 )
   {
      HAL_UART_Receive( ( UARTn_Type* )UART1, &spiTx_ch, 1, BLOCKING );

      HAL_USART_Transmit( ( USART1n_Type* )USART10, &spiTx_ch, 1, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )"\r\nUSRT10 tx:", 12, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, &spiTx_ch, 1, BLOCKING );

      HAL_USART_Receive( ( USART1n_Type* )USART11, &spiRx_ch, 1, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, ( uint8_t* )", USRT11 rx:", 12, BLOCKING );
      HAL_UART_Transmit( ( UARTn_Type* )UART1, &spiRx_ch, 1, BLOCKING );
   }
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main loop
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void mainloop( void )
{
   /* Configure menu prinf */
   DEBUG_MenuPrint();

   /* USRT11_Configure */
   USRT11_Configure();

   /* USRT10_Configure */
   USRT10_Configure();

   /* USART1n_USRT_PollingRun */
   USART1n_USRT_PollingRun();
}

/*-------------------------------------------------------------------------*//**
 * @brief         Main program
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
int main( void )
{
   /* Initialize all port */
   Port_Init();

   /* Configure the system clock to HSI 40MHz */
   SystemClock_Config();

   /* Initialize Debug frame work through initializing USART port  */
   DEBUG_Init();

   /* Infinite loop */
   mainloop();

   return 0;
}

/*-------------------------------------------------------------------------*//**
 * @brief         Driver error handler
 * @param         None
 * @return        None
 *//*-------------------------------------------------------------------------*/
void Error_Handler( void )
{
   while( 1 )
   {
   }
}

#ifdef  USE_FULL_ASSERT
/*-------------------------------------------------------------------------*//**
 * @brief         Reports the name of the source file and the source line number
 *                where the assert_param error has occurred.
 * @param         file: pointer to the source file name
 * @param         line: assert_param error line source number
 * @return        None
 *//*-------------------------------------------------------------------------*/
void check_failed( uint8_t* file, uint32_t line )
{
   /* User can add his own implementation to report the file name and line number,
      ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while( 1 )
   {
   }
}
#endif

